# -*- coding: utf-8 -*-
# Copyright 2019 Ildar.Shirshov <ildar-shirshov@ya.ru>
#  
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

__version__ = "0.0.1a2"
__doc__ = """
"""
__author__ = "Ildar Shirshov"
__license__ = "Apache 2.0"

from . import routes
from . import adaptor
from . import generators
